!####################################################################################
!                          TEST PROGRAM
!####################################################################################
MODULE hhh
CONTAINS

SUBROUTINE func1(x,fval,fderiv)
!
!    function used in root finding
!
  USE nrtype
  IMPLICIT NONE
  REAL(SP), INTENT(IN) :: x
  REAL(SP), INTENT(OUT) :: fval, fderiv

  fval   =   COS(x)
  fderiv = - SIN(x)

END SUBROUTINE func1

!********************************************************************************************

SUBROUTINE derivs(x,y,dydx)
!
!   differential equation
!
USE nrtype
IMPLICIT NONE
REAL(SP), INTENT(IN) :: x
REAL(SP), DIMENSION(:), INTENT(IN) :: y
REAL(SP), DIMENSION(:), INTENT(OUT) :: dydx

dydx(1) =   y(2)
dydx(2) = - y(1)

END SUBROUTINE derivs

END MODULE

PROGRAM test
  !---------------------------------------------------------------------------
  !    simple test program for NR routines
  !---------------------------------------------------------------------------
  !
  !   compilation hints:
  !
  !                 1.1) compile nrtype.f nr.f nrutil.f creating nrtype.o nr.o nrutil.o and .mod-files:
  !
  !                           ifort -c nrtype.f90 nr.f90 nrutil.f90 recipes.f90
  !
  !                 1.2) compile main program by (e.g.)
  !
  !                           ifort nrtype.o nr.o nrutil.o recipes.o test.f90  
  !
  !          alternatively, you might use
  !
  !                 2.)   ifort nrtype.f90 nr.f90 nrutil.f90 recipes.f90 test.f90 
  !
  !                    (this takes longer, since all routines are recompiled)
  !
  !          Finally, there is a Makefile which does all compilation for you, just type:
  !  
  !                 3) make
  !
  !
  !---------------------------------------------------------------------------
  !                        written by Ch. Theis (May 28, 2008)
  !---------------------------------------------------------------------------
  USE nrtype
  USE nrutil
  USE nr
  USE ode_path
  USE hhh
  !
  IMPLICIT NONE
  INTEGER(I4B) :: i,nsteps
  REAL(SP) :: xlow,xhig,dx,xacc,root
  REAL(SP) :: eps,h1,hmin,relerr,x1,x2,yexact
  REAL(SP), DIMENSION(2) :: ystart

  EXTERNAL func1      ! for rtsafe
  EXTERNAL derivs     ! for odeint

!---------------------------------------------------------------------
!   integration of the wave equation: tests odeint
!---------------------------------------------------------------------

  nsteps = 10
!
  xlow = 0.D0
  xhig = 10.D0
  dx   = (xhig - xlow) / DBLE(nsteps)
!
!    parameters for odeint
!
  eps  = 1.D-8
  h1   = 0.1D0
  hmin = 0.D0
!
!    initial values  
!
  x2 = 0.D0
  ystart(1) = SIN(x2)
  ystart(2) = COS(x2)
!
  PRINT*,' '
  PRINT*,'  Runge-Kutta integration'
  PRINT*,' '
  PRINT*,'    x,                          f(x),            relative deviation'
  PRINT*,' '
  DO i=1,nsteps

!           prepare for next step
     x1 = x2
     x2 = x2 + dx

!           do the integration
     CALL odeint(ystart,x1,x2,eps,h1,hmin,derivs,rkqs)

     yexact = SIN(x2)
     relerr = ABS(ystart(1) - yexact) / ABS(yexact)
     PRINT*,x2,ystart(1),relerr

  ENDDO
  PRINT*,' '

!---------------------------------------------------------------------
!   root finding of COS(x): tests rtsafe
!---------------------------------------------------------------------


END PROGRAM test

!********************************************************************************************


