      PROGRAM parenc
C------------------------------------------------------------
C--
C--      restricted N-body solver for parabolic encounter
C--
C------------------------------------------------------------
C--
C--      used variables:
C--      ==============
C--
C------------------------------------------------------------
C--             date: 
C--------------------------------------------------------------------------
C-- !!  NOTE: uncomment and complete the calls to the subroutine outbod !!
C--------------------------------------------------------------------------
      
      INCLUDE 'parenc.inc'
      DOUBLE PRECISION cpu0,cpu1


C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C--    A) START RUN
C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++

C---------------------------------
C--          get actual cputime
C---------------------------------
      CALL cputime(cpu0)
C          -------

C--------------------------------------------------------------------
C--       open output file and save initial status
C--------------------------------------------------------------------
Coutbod      CALL outbod(0,... for additional parameters see the SUBROUTINE outbod)
C          ------


C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C--  B)  PERFORM THE SIMULATION
C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      CALL orbit
C          -----
      
C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C--  C)  FINISH RUN
C--+++++++++++++++++++++++++++++++++++++++++++++++++++++++++

C---------------------------------
C--          used cputime
C---------------------------------
      CALL cputime(cpu1)
C          -------
      xhelp = cpu1 - cpu0
      WRITE(*,900) xhelp
900   FORMAT(1x,' used CPU-time (sec)',f10.1,2x)

C--------------------------------------------------------------------
C--       finish output and close output files
C--------------------------------------------------------------------
Coutbod      CALL outbod(2,... for additional parameters see the SUBROUTINE outbod)
C          ------

      S T O P
      END

C********************************************************************

      SUBROUTINE orbit
C----------------------------------------------------------------------
C-- this subroutine should calculate the orbits of the test particles
C----------------------------------------------------------------------
      INCLUDE 'parenc.inc'
      DOUBLE PRECISION hnext
      INTEGER kmax
C--        this COMMON block is used for odeint
      COMMON /path/ hnext,kmax

C--        set kmax=0 (cf. odeint)
      kmax=0


C--             call outbod whenever you want the positions of the particles
Coutbod      CALL outbod(1,... for additional parameters see the SUBROUTINE outbod)
C          ------


      R E T U R N
      END

C********************************************************************

      SUBROUTINE outbod(imod,iform,nb1,nb2,nbmax,tnow,pos)
C------------------------------------------------------------------
C--        OUTPUT TO BODY DATA FILE (outbo.bin)
C------------------------------------------------------------------
C--
C--    NOTE:
C--    =====
C--
C--        1) the output-file uses channel 42
C--        2) outbod is written to 'outbo.bin' or 'outbo.asc'
C--        3) the floating point numbers must be DOUBLE PRECISION
C--
C-------------------------------------------------------------------
C--   
C--    INPUT PARAMETER:
C--    ===============
C--
C--         imod:  = 0 open output file
C--                = 1 unformatted output
C--                = 2 finish output
C--         iform: = 0 unformatted output
C--                = 1 formatted output
C--         nb1:   = number of test particles in galaxy 1
C--         nb2:   = number of test particles in galaxy 2
C--         nbmax: = dimension of the position array of test particles
C--                   (NOTE: this value MUST match the declaration of the
C--                          array used for pos in the call of outbod)
C--         tnow:  = actual time
C--         pos:   = positions of test particels
C--
C-----------------------------------------------------
C--       date: February 12, 1998  (Christian Theis)
C-----------------------------------------------------

      INTEGER imod,iform,i,j,iobod,nbody,ndim,nbmax,nb1,nb2
      DOUBLE PRECISION tnow,pos(1:nbmax,1:3)
      PARAMETER(iobod=42,ndim=3)

C--             total number of test particles
      nbody = nb1 + nb2

C-------------------------------------------------------
C--             open output file
C-------------------------------------------------------
      IF(imod.EQ.0) THEN
         IF(iform.EQ.0) THEN
            OPEN(iobod,FILE='outbo.bin',STATUS='unknown',
     &                 FORM='unformatted')
         ELSE
            OPEN(iobod,FILE='outbo.asc',STATUS='unknown',
     &                 FORM='formatted')
         ENDIF
      ENDIF

C-------------------------------------------------------
C--             write output
C-------------------------------------------------------
      IF(imod.EQ.1 .OR. imod.EQ.0) THEN

C--        unformatted output
C          ------------------
         IF(iform.EQ.0) THEN
            WRITE(iobod) nbody,nb1,nb2,ndim,tnow
            WRITE(iobod) ((pos(i,j),i=1,nbody),j=1,3)
         ELSE

C--        formatted output
C          ----------------
            WRITE(iobod,110) nbody,nb1,nb2
            WRITE(iobod,120) ndim
            WRITE(iobod,130) tnow
            DO 100 i=1,nbody
               WRITE(iobod,140) (pos(i,j),j=1,3)
100         CONTINUE
 
110         FORMAT(5X,3(I10,2X),14X,' NBODIES')
120         FORMAT(12X,I3,40X,' NDIM')
130         FORMAT(5X,1P,E10.3,40X,' TNOW')
140         FORMAT(1P,3(E12.4,1X))
         ENDIF
      ENDIF 
 
C-------------------------------------------------------
C--             finish output
C-------------------------------------------------------
      IF(imod.EQ.2) THEN
         CLOSE(iobod)
      ENDIF

      R E T U R N
      END

C***********************************************************************
C
C
                    SUBROUTINE cputime(actcpu)
C
C                        date: 9/28/95
C
C***********************************************************************
C
C   ACTCPU : total CPU-time for the job
C
C***********************************************************************

      DOUBLE PRECISION actcpu,dummy

      REAL*4 cpusun(2)

C---------------------------------
C--         SPARCstation
C--       CPUSUN(1): user time
C--       CPUSUN(2): system time
C---------------------------------

      dummy  = ETIME(cpusun)
      actcpu = cpusun(1) + cpusun(2)

      R E T U R N
      END

C#####################################################################

      SUBROUTINE odeint(ystart,nvar,x1,x2,eps,h1,hmin,nok,nbad,derivs,
     *rkqs)
C------------------------------------------------------------
C--      solves system of ordinary differential equations
C--         with an adaptive Runge-Kutta integrator
C--       (for details cf. to 'Numerical Recipes' p. 714)
C------------------------------------------------------------
C--       !!      modified in /path/ (hnext)  !!
C------------------------------------------------------------
C--        
C--        VARIABLES:
C--        =========
C--        
C--          1)  INPUT:
C--        
C--              ystart(1:nvar):  initial values
C--              nvar:            number of equations
C--              x1,x2:           start and end point of integration
C--              eps:             accuracy of integration
C--              h1:              first estimate for stepsize
C--              hmin:            lower limit for stepsize (can be zero)
C--              derivs:          user supplied system of differential
C--                               equations; must be in the form
C--                       subroutine derivs(x,y,dydx) with the
C--                       DOUBLE PRECISION variables x,y(nvar),dydx(nvar)
C--                       calculating the function values y(.) and their
C--                       derivatives dydx(.) at position x
C--              rkqs:            Runke-Kutta driver (don't modify this!)
C--                 NOTE: both, derivs and rkqs, must be declared as EXTERNAL
C--                       
C--          2)  OUTPUT:
C--        
C--              ystart(1:nvar):  final values of the functions at x2
C--              nok,nbad:        number of good and bad steps
C--              hnext:           estimated next step size (NOTE: this
C--                               value is given in the path and can
C--                               be used for h1 in the next call of odeint!)
C--        
C--####################################################################
C--        NOTE:  you must define a COMMON block path in the calling
C--               routine and you must set kmax to zero here!
C--####################################################################

      INTEGER nbad,nok,nvar,KMAXX,MAXSTP,NMAX
      DOUBLE PRECISION eps,h1,hmin,x1,x2,ystart(nvar),TINY
      EXTERNAL derivs,rkqs
      PARAMETER (MAXSTP=10000,NMAX=50,KMAXX=200,TINY=1.d-30)
      INTEGER i,kmax,kount,nstp
      DOUBLE PRECISION dxsav,h,hdid,hnext,x,xsav,dydx(NMAX),xp(KMAXX),y
     *(NMAX),
     *yp(NMAX,KMAXX),yscal(NMAX)
      COMMON /path/ hnext,kmax
      x=x1
      h=sign(h1,x2-x1)
      nok=0
      nbad=0
      kount=0
      do 11 i=1,nvar
        y(i)=ystart(i)
11    continue
      if (kmax.gt.0) xsav=x-2.d0*dxsav
      do 16 nstp=1,MAXSTP
        call derivs(x,y,dydx)
        do 12 i=1,nvar
          yscal(i)=abs(y(i))+abs(h*dydx(i))+TINY
12      continue
        if(kmax.gt.0)then
          if(abs(x-xsav).gt.abs(dxsav)) then
            if(kount.lt.kmax-1)then
              kount=kount+1
              xp(kount)=x
              do 13 i=1,nvar
                yp(i,kount)=y(i)
13            continue
              xsav=x
            endif
          endif
        endif
        if((x+h-x2)*(x+h-x1).gt.0.d0) h=x2-x
        call rkqs(y,dydx,nvar,x,h,eps,yscal,hdid,hnext,derivs)
        if(hdid.eq.h)then
          nok=nok+1
        else
          nbad=nbad+1
        endif

        if((x-x2)*(x2-x1).ge.0.d0) then
          do 14 i=1,nvar
            ystart(i)=y(i)
14        continue
          if(kmax.ne.0)then
            kount=kount+1
            xp(kount)=x
            do 15 i=1,nvar
              yp(i,kount)=y(i)
15          continue
          endif
          return
        endif
        if(abs(hnext).lt.hmin) pause
     *'stepsize smaller than minimum in odeint'
        h=hnext
16    continue
      pause 'too many steps in odeint'
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 5.d0).

C#####################################################################

      SUBROUTINE rkqs(y,dydx,n,x,htry,eps,yscal,hdid,hnext,derivs)
C----------------------------
C--       used for odeint
C----------------------------
      INTEGER n,NMAX
      DOUBLE PRECISION eps,hdid,hnext,htry,x,dydx(n),y(n),yscal(n)
      EXTERNAL derivs
      PARAMETER (NMAX=50)
CU    USES derivs,rkck
      INTEGER i
      DOUBLE PRECISION errmax,h,xnew,yerr(NMAX),ytemp(NMAX),SAFETY,PGROW
     *,PSHRNK,
     *ERRCON
      PARAMETER (SAFETY=0.9d0,PGROW=-.2d0,PSHRNK=-.25d0,ERRCON=1.89d-4)
      h=htry
1     call rkck(y,dydx,n,x,h,ytemp,yerr,derivs)
      errmax=0.d0
      do 11 i=1,n
        errmax=max(errmax,abs(yerr(i)/yscal(i)))
11    continue
      errmax=errmax/eps
      if(errmax.gt.1.d0)then
        h=SAFETY*h*(errmax**PSHRNK)
        if(h.lt.0.1d0*h)then
          h=.1d0*h
        endif
        xnew=x+h
        if(xnew.eq.x)pause 'stepsize underflow in rkqs'
        goto 1
      else
        if(errmax.gt.ERRCON)then
          hnext=SAFETY*h*(errmax**PGROW)
        else
          hnext=5.d0*h
        endif
        hdid=h
        x=x+h
        do 12 i=1,n
          y(i)=ytemp(i)
12      continue
        return
      endif
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 5.d0).

C#####################################################################

      SUBROUTINE rkck(y,dydx,n,x,h,yout,yerr,derivs)
C----------------------------
C--       used for odeint
C----------------------------
      INTEGER n,NMAX
      DOUBLE PRECISION h,x,dydx(n),y(n),yerr(n),yout(n)
      EXTERNAL derivs
      PARAMETER (NMAX=50)
CU    USES derivs
      INTEGER i
      DOUBLE PRECISION ak2(NMAX),ak3(NMAX),ak4(NMAX),ak5(NMAX),ak6(NMAX)
     *,
     *ytemp(NMAX),A2,A3,A4,A5,A6,B21,B31,B32,B41,B42,B43,B51,B52,B53,
     *B54,B61,B62,B63,B64,B65,C1,C3,C4,C6,DC1,DC3,DC4,DC5,DC6
      PARAMETER (A2=.2d0,A3=.3d0,A4=.6d0,A5=1.d0,A6=.875d0,B21=.2d0,B31
     *=3.d0/40.d0,
     *B32=9.d0/40.d0,B41=.3d0,B42=-.9d0,B43=1.2d0,B51=-11.d0/54.d0,B52
     *=2.5d0,
     *B53=-70.d0/27.d0,B54=35.d0/27.d0,B61=1631.d0/55296.d0,B62=175.d0
     */512.d0,
     *B63=575.d0/13824.d0,B64=44275.d0/110592.d0,B65=253.d0/4096.d0,C1
     *=37.d0/378.d0,
     *C3=250.d0/621.d0,C4=125.d0/594.d0,C6=512.d0/1771.d0,DC1=C1-2825.d0
     */27648.d0,
     *DC3=C3-18575.d0/48384.d0,DC4=C4-13525.d0/55296.d0,DC5=-277.d0
     */14336.d0,
     *DC6=C6-.25d0)
      do 11 i=1,n
        ytemp(i)=y(i)+B21*h*dydx(i)
11    continue
      call derivs(x+A2*h,ytemp,ak2)
      do 12 i=1,n
        ytemp(i)=y(i)+h*(B31*dydx(i)+B32*ak2(i))
12    continue
      call derivs(x+A3*h,ytemp,ak3)
      do 13 i=1,n
        ytemp(i)=y(i)+h*(B41*dydx(i)+B42*ak2(i)+B43*ak3(i))
13    continue
      call derivs(x+A4*h,ytemp,ak4)
      do 14 i=1,n
        ytemp(i)=y(i)+h*(B51*dydx(i)+B52*ak2(i)+B53*ak3(i)+B54*ak4(i))
14    continue
      call derivs(x+A5*h,ytemp,ak5)
      do 15 i=1,n
        ytemp(i)=y(i)+h*(B61*dydx(i)+B62*ak2(i)+B63*ak3(i)+B64*ak4(i)+
     *B65*ak5(i))
15    continue
      call derivs(x+A6*h,ytemp,ak6)
      do 16 i=1,n
        yout(i)=y(i)+h*(C1*dydx(i)+C3*ak3(i)+C4*ak4(i)+C6*ak6(i))
16    continue
      do 17 i=1,n
        yerr(i)=h*(DC1*dydx(i)+DC3*ak3(i)+DC4*ak4(i)+DC5*ak5(i)+DC6*
     *ak6(i))
17    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 5.d0).

C***********************************************************************

      FUNCTION rtsafe(funcd,x1,x2,xacc)
C-----------------------------------------------------------------
C-- modified for relative error (Jan 25, 1997, CPT)
C-----------------------------------------------------------------
C--
C--        VARIABLES:
C--        =========
C--        
C--          1)  INPUT:    
C--
C--                 funcd:   function to be solved funcd(x)=0 
C--                 x1,x2:   values bracketing the root
C--                 xacc:    relative error of root determination
C--
C--          2)  OUTPUT:
C--
C--                 rtsafe:  root
C--
C-----------------------------------------------------------------
      INTEGER MAXIT
      DOUBLE PRECISION rtsafe,x1,x2,xacc,tiny
      EXTERNAL funcd
      PARAMETER (MAXIT=100,tiny=1.D-30)
      INTEGER j
      DOUBLE PRECISION df,dx,dxold,f,fh,fl,temp,xh,xl
      call funcd(x1,fl,df)
      call funcd(x2,fh,df)
      if((fl.gt.0.d0.and.fh.gt.0.d0).or.(fl.lt.0.d0.and.fh.lt.0.d0)
     *)pause
     *'root must be bracketed in rtsafe'
      if(fl.eq.0.d0)then
        rtsafe=x1
        return
      else if(fh.eq.0.d0)then
        rtsafe=x2
        return
      else if(fl.lt.0.d0)then
        xl=x1
        xh=x2
      else
        xh=x1
        xl=x2
      endif
      rtsafe=.5d0*(x1+x2)
      dxold=abs(x2-x1)
      dx=dxold
      call funcd(rtsafe,f,df)
      do 11 j=1,MAXIT
        if(((rtsafe-xh)*df-f)*((rtsafe-xl)*df-f).ge.0.d0.or. abs(2.d0*
     *f).gt.abs(dxold*df) ) then
          dxold=dx
          dx=0.5d0*(xh-xl)
          rtsafe=xl+dx
          if(xl.eq.rtsafe)return
        else
          dxold=dx
          dx=f/df
          temp=rtsafe
          rtsafe=rtsafe-dx
          if(temp.eq.rtsafe)return
        endif
        if(abs(dx/(ABS(rtsafe)+tiny)).lt.xacc) return
        call funcd(rtsafe,f,df)
        if(f.lt.0.d0) then
          xl=rtsafe
        else
          xh=rtsafe
        endif
11    continue
      pause 'rtsafe exceeding maximum iterations'
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software 5.d0).

